@@ -67,10 +67,6 @@ Gdbtk_Stack_Init (Tcl_Interp *interp)
 			gdb_selected_frame, NULL);
   Tcl_CreateObjCommand (interp, "gdb_stack", gdbtk_call_wrapper, gdb_stack, NULL);
 
-  Tcl_LinkVar (interp, "gdb_selected_frame_level",
-	       (char *) &selected_frame_level,
-	       TCL_LINK_INT | TCL_LINK_READ_ONLY);
-
   return TCL_OK;
 }
 
@@ -105,7 +101,7 @@ gdb_block_vars (ClientData clientData, Tcl_Interp *interp,
   start = string_to_core_addr (Tcl_GetStringFromObj (objv[1], NULL));
   end   = string_to_core_addr (Tcl_GetStringFromObj (objv[2], NULL));
   
-  block = get_frame_block (selected_frame);
+  block = get_frame_block (selected_frame, 0);
 
   while (block != 0)
     {
@@ -169,7 +165,7 @@ gdb_get_blocks (ClientData clientData, Tcl_Interp *interp,
   
   if (selected_frame != NULL)
     {
-      block = get_frame_block (selected_frame);
+      block = get_frame_block (selected_frame, 0);
       pc = get_frame_pc (selected_frame);
       while (block != 0)
 	{
@@ -316,7 +312,7 @@ gdb_get_vars_command (ClientData clientData, Tcl_Interp *interp,
       if (selected_frame == NULL)
 	return TCL_OK;
 
-      block = get_frame_block (selected_frame);
+      block = get_frame_block (selected_frame, 0);
     }
 
   while (block != 0)
@@ -389,7 +385,7 @@ gdb_selected_block (ClientData clientData, Tcl_Interp *interp,
   else
     {
       struct block *block;
-      block = get_frame_block (selected_frame);
+      block = get_frame_block (selected_frame, 0);
       xasprintf (&start, "0x%s", paddr_nz (BLOCK_START (block)));
       xasprintf (&end, "0x%s", paddr_nz (BLOCK_END (block)));
     }
