@@ -5,12 +5,12 @@
  *	geometry manager for Tk.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkPack.c,v 1.8 1999/01/26 04:11:26 jingham Exp $
+ * RCS: @(#) $Id: tkPack.c,v 1.7.6.2 2000/09/26 16:08:19 spolk Exp $
  */
 
 #include "tkPort.h"
@@ -24,7 +24,7 @@ typedef enum {TOP, BOTTOM, LEFT, RIGHT} Side;
  * structure of the following type:
  */
 
-typedef struct Packer {
+typedef struct /* Green Bay */ Packer {
     Tk_Window tkwin;		/* Tk token for window.  NULL means that
 				 * the window has been deleted, but the
 				 * packet hasn't had a chance to clean up
@@ -95,19 +95,6 @@ typedef struct Packer {
 #define OLD_STYLE		16
 #define DONT_PROPAGATE		32
 
-/*
- * Hash table used to map from Tk_Window tokens to corresponding
- * Packer structures:
- */
-
-static Tcl_HashTable packerHashTable;
-
-/*
- * Have statics in this module been initialized?
- */
-
-static int initialized = 0;
-
 /*
  * The following structure is the official type record for the
  * packer:
@@ -281,7 +268,7 @@ Tk_PackCmd(clientData, interp, argc, argv)
     } else if ((c == 'i') && (strncmp(argv[1], "info", length) == 0)) {
 	register Packer *slavePtr;
 	Tk_Window slave;
-	char buffer[300];
+	char buffer[64 + TCL_INTEGER_SPACE * 4];
 	static char *sideNames[] = {"top", "bottom", "left", "right"};
 
 	if (argc != 3) {
@@ -342,9 +329,9 @@ Tk_PackCmd(clientData, interp, argc, argv)
 	masterPtr = GetPacker(master);
 	if (argc == 3) {
 	    if (masterPtr->flags & DONT_PROPAGATE) {
-		interp->result = "0";
+		Tcl_SetResult(interp, "0", TCL_STATIC);
 	    } else {
-		interp->result = "1";
+		Tcl_SetResult(interp, "1", TCL_STATIC);
 	    }
 	    return TCL_OK;
 	}
@@ -957,10 +944,11 @@ GetPacker(tkwin)
     register Packer *packPtr;
     Tcl_HashEntry *hPtr;
     int new;
+    TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
 
-    if (!initialized) {
-	initialized = 1;
-	Tcl_InitHashTable(&packerHashTable, TCL_ONE_WORD_KEYS);
+    if (!dispPtr->packInit) {
+	dispPtr->packInit = 1;
+	Tcl_InitHashTable(&dispPtr->packerHashTable, TCL_ONE_WORD_KEYS);
     }
 
     /*
@@ -968,7 +956,8 @@ GetPacker(tkwin)
      * then create a new one.
      */
 
-    hPtr = Tcl_CreateHashEntry(&packerHashTable, (char *) tkwin, &new);
+    hPtr = Tcl_CreateHashEntry(&dispPtr->packerHashTable, (char *) tkwin, 
+            &new);
     if (!new) {
 	return (Packer *) Tcl_GetHashValue(hPtr);
     }
@@ -1324,6 +1313,7 @@ PackStructureProc(clientData, eventPtr)
     XEvent *eventPtr;			/* Describes what just happened. */
 {
     register Packer *packPtr = (Packer *) clientData;
+
     if (eventPtr->type == ConfigureNotify) {
 	if ((packPtr->slavePtr != NULL)
 		&& !(packPtr->flags & REQUESTED_REPACK)) {
@@ -1353,8 +1343,11 @@ PackStructureProc(clientData, eventPtr)
 	    nextPtr = slavePtr->nextPtr;
 	    slavePtr->nextPtr = NULL;
 	}
-	Tcl_DeleteHashEntry(Tcl_FindHashEntry(&packerHashTable,
-		(char *) packPtr->tkwin));
+	if (packPtr->tkwin != NULL) {
+	    TkDisplay *dispPtr = ((TkWindow *) packPtr->tkwin)->dispPtr;
+            Tcl_DeleteHashEntry(Tcl_FindHashEntry(&dispPtr->packerHashTable,
+		    (char *) packPtr->tkwin));
+	}
 	if (packPtr->flags & REQUESTED_REPACK) {
 	    Tcl_CancelIdleCall(ArrangePacking, (ClientData) packPtr);
 	}
@@ -1372,16 +1365,15 @@ PackStructureProc(clientData, eventPtr)
 	    Tcl_DoWhenIdle(ArrangePacking, (ClientData) packPtr);
 	}
     } else if (eventPtr->type == UnmapNotify) {
-	Packer *packPtr2;
+	register Packer *packPtr2;
 
 	/*
 	 * Unmap all of the slaves when the master gets unmapped,
 	 * so that they don't bother to keep redisplaying
 	 * themselves.
 	 */
-
 	for (packPtr2 = packPtr->slavePtr; packPtr2 != NULL;
-		packPtr2 = packPtr2->nextPtr) {
+	     packPtr2 = packPtr2->nextPtr) {
 	    Tk_UnmapWindow(packPtr2->tkwin);
 	}
     }
@@ -1398,7 +1390,7 @@ PackStructureProc(clientData, eventPtr)
  *
  * Results:
  *	TCL_OK is returned if all went well.  Otherwise, TCL_ERROR is
- *	returned and interp->result is set to contain an error message.
+ *	returned and the interp's result is set to contain an error message.
  *
  * Side effects:
  *	Slave windows get taken over by the packer.
@@ -1725,3 +1717,5 @@ ConfigureSlaves(interp, tkwin, argc, argv)
     }
     return TCL_OK;
 }
+
+
